# -*- coding: utf-8 -*-

import re
import base64
from odoo.addons.saas_worker.compat import route, http, request
from odoo import Command, tools
from functools import reduce

class SupportForm(http.Controller):

    @route('/help', auth='public', website=True, sitemap=True, readonly=True)
    def report_issue(self, **kwargs):
        previously_entered_values = kwargs
        # Fetch information from existing ticket, if any
        if 'ticket_id' in kwargs and request.env.user != request.env.ref('base.public_user'):
            ticket_id = request.env['project.task'].search([('id', '=', kwargs['ticket_id'])])

            if ticket_id:
                attachments = request.env['ir.attachment'].search(
                    [('res_model', '=', 'project.task'), ('res_id', '=', ticket_id.id)]
                )
                mail_values = dict(
                    summary=ticket_id.name,
                    description_previous=ticket_id.description,
                    attachments=attachments
                )
                # What users just wrote in their previous submit attempt still
                # takes precedence over the content of their original email
                mail_values.update(previously_entered_values)
                previously_entered_values = mail_values

        # If user is logged, pre-fill some fields
        if request.env.uid != request.env.ref('base.public_user').id:
            instances = request.env['openerp.enterprise.database']._get_instances(alive=True)
            instances = [i for i in instances if i['visibility'] not in ('hidden', 'support')]
            partner = request.env.user.partner_id
            contracts = request.env['sale.order'].sudo().search([
                ('is_subscription', '=', True),
                '|', '|',
                ('partner_id', 'parent_of', partner.id),
                ('referrer_id', 'parent_of', partner.id),
                ('enterprise_final_customer_id', 'parent_of', partner.id),
                ('subscription_state', 'in', ['3_progress', '4_paused'])
            ])
            guess_online = kwargs.get('edition') == 'online' or instances or 'dbname' in kwargs.get('errors', [])
            guess_enterprise = kwargs.get('edition') == 'enterprise' or contracts or 'contract_number' in kwargs.get('errors', [])
            # get_contracts

            prefill_values = dict(
                name=partner.name,
                email=partner.email,
                phone=partner.mobile or partner.phone,
                instances=bool(instances),
                paid_instances=[db for db in instances if db['is_paid']],
                free_instances=[db for db in instances if not db['is_paid']],
                contracts=contracts,
                edition=guess_online and 'online' or guess_enterprise and 'enterprise',
            )
            # What users just wrote in their previous submit attempt still
            # takes precedence over the content of their original email, which
            # itself takes precedence over the default and guessed values.
            prefill_values.update(previously_entered_values)
            previously_entered_values = prefill_values

            if request.env.user.partner_id.commercial_partner_id.grade_id:
                previously_entered_values['pack_tasks'] = request.env['project.task'].sudo().search([
                    ('partner_id', 'child_of', partner.commercial_partner_id.id),
                    ('project_id', '=', request.env.ref('openerp_enterprise.project_success_pack_be').id),
                    ('stage_id.fold', '=', False)
                ], order='id asc')

        # Add a 'reproduced on runbot' for employees (internal users)
        # In order to  directly assign the ticket in the bugfix stage
        previously_entered_values.update(internal_user=not request.env.user.share)

        return request.render('openerp_enterprise.support_form', previously_entered_values)

    @route('/help/submit', auth='public', methods=['POST'], website=True)
    def report_issue_submit(self, **kwargs):
        required_fields = ['name', 'email', 'edition', 'summary', 'description', 'stage']
        contextual_fields = ['phone', 'edition', 'dbname', 'extra_dbname',
                             'version', 'contract_number', 'attachments']
        edition = kwargs.get('edition')
        if edition == 'online':
            if kwargs.get('dbname', 'other') == 'other':
                required_fields.extend(['extra_dbname'])
            else:
                required_fields.extend(['dbname'])
                kwargs.pop('extra_dbname', None)
            for f in ['contract_number']:
                kwargs.pop(f, None)
        elif edition != 'partner_pack':
            required_fields.extend(['contract_number'])
            if kwargs.get('contract_number', 'other') == 'other':
                required_fields.remove('contract_number')
                required_fields.extend(['contract_number_other'])
            kwargs.pop('dbname', None)

        missing_fields = [f for f in required_fields if not kwargs.get(f)]
        if missing_fields:
            return self.report_issue(**dict(kwargs, errors=missing_fields))

        values = {
            'name': kwargs['summary'],
            'user_ids': False,  # Unassigned by default, otherwise it is assigned to the creator
            'tag_ids': [],
        }

        if edition == 'online':
            # Check database name and use the associated partner
            db_name = kwargs.get('extra_dbname') or kwargs.get('dbname')
            db_ids = self._check_db_name(db_name)
            if db_name == 'openerp':
                tag_id = request.env.ref('openerp_enterprise.project_tag_internal')
                values['tag_ids'] = [Command.link(tag_id.id)]
            elif len(db_ids) == 1:
                database_id = request.env['openerp.enterprise.database'].sudo().browse(db_ids[0])
                kwargs['version'] = database_id.version
                values['mnt_subscription_id'] = (
                    database_id.subscription_id.id or
                    database_id.parent_id.subscription_id.id)
            elif not db_ids:
                # RAR: temporary measure to reduce the number of incoming tickets
                return self.report_issue(**dict(kwargs, errors=['extra_dbname']))
            else:
                # More then one DB, try to find the contract of alive database
                # if there is only one proceed
                databases = request.env['openerp.enterprise.database'].sudo().browse(db_ids)
                main_databases = databases.filtered(lambda db:
                    db.alive and (db.subscription_id or db.parent_id.subscription_id)
                )
                if len(main_databases) == 1:
                    values['mnt_subscription_id'] = (
                        main_databases.subscription_id.id or
                        main_databases.parent_id.subscription_id.id)
        elif edition != 'partner_pack':
            if kwargs.get('contract_number', 'other') == 'other':
                contract_number = kwargs.get('contract_number_other')
            else:
                contract_number = kwargs.get('contract_number')
            # Check contract number
            SaleOrder = request.env['sale.order'].sudo()
            contract_id = SaleOrder._get_most_valid_subscription(contract_number).filtered(
                lambda c: not c.enterprise_is_partnership
            )
            if not contract_id:
                return self.report_issue(**dict(kwargs, errors=['contract_number']))
            else:
                contract_id = contract_id[0]
                values['mnt_subscription_id'] = contract_id.id
                database_id = request.env['openerp.enterprise.database'].sudo()\
                    .search([('subscription_id', '=', contract_id.id)], limit=1)
                kwargs['version'] = database_id.version
            if contract_id.subscription_state not in ['3_progress', '4_paused'] and contract_id.user_id.email:
                kwargs['account_manager_email'] = contract_id.user_id.email
                return self.report_issue(**dict(kwargs, errors=['contract_number']))

        follower_ids = []
        partner_id = None
        if request.env.uid != request.env.ref('base.public_user').id:
            # If user was logged, use his partner_id
            partner_id = request.env.user.partner_id
        else:
            # Try to find a partner with the given email address
            partner_id = request.env['res.partner'].sudo().search([('email', '=', kwargs['email']), ('type', '!=', 'private')])
            if partner_id:
                partner_id = partner_id[0]

        if not partner_id:
            partner_id = request.env['res.partner'].sudo().create({
                'email': kwargs['email'],
                'name': kwargs['email'],
            })

        values['partner_id'] = partner_id.id
        follower_ids.append(partner_id.id)

        stage = kwargs.get('stage')
        project_id = None
        if stage == 'migration':
            # project: Upgrade Issues
            project_id = request.env.ref('openerp_enterprise.project_mig_issues_l1')
        elif edition == 'partner_pack':
            project_id = request.env.ref('openerp_enterprise.project_partner_success_new_request')
            kwargs['description'] = kwargs['description_partner_pack']
            parent_task = request.env["project.task"].browse(int(kwargs.get('pack_task'))).exists()
            assert parent_task in request.env['project.task'].sudo().search([
                ('partner_id', 'child_of', partner_id.commercial_partner_id.id),
                ('project_id', '=', request.env.ref('openerp_enterprise.project_success_pack_be').id),
                ('stage_id.fold', '=', False)
            ])
            values['parent_id'] = parent_task.id
            values['sale_line_id'] = parent_task.sudo().sale_line_id.id
            values['stage_id'] = request.env.ref('openerp_enterprise.project_stage_partner_success_new').id
        else:
            # project: Help
            project_id = request.env.ref('openerp_enterprise.project_help')
        values['project_id'] = project_id.id

        tag_id = None
        if stage == 'new' and not request.env.user.share and kwargs.get('reproducible') == 'on':
            tag_id = request.env.ref('openerp_enterprise.project_tag_bug_fix')
        elif stage == 'blocked':
            values['stage_id'] = request.env.ref('openerp_enterprise.stage_admin_help').id
            values['priority'] = '1'
        elif stage == 'administration':
            values['stage_id'] = request.env.ref('openerp_enterprise.stage_admin_help').id
            tag_id = request.env.ref('openerp_enterprise.project_tag_administration')
        elif stage == 'migration':
            kwargs['description'] = kwargs['description_migration']
        elif stage == 'company_info':
            tag_id = request.env.ref('openerp_enterprise.tag_company_data_update')
            kwargs['description'] = kwargs['description_companyinfo']
            values['stage_id'] = request.env.ref('openerp_enterprise.stage_admin_help').id
        elif stage == 'post_upgrade':
            tag_id = request.env.ref('openerp_enterprise.tag_post_upgrade')
        elif stage == 'bank_sync':
            tag_id = request.env.ref('openerp_enterprise.tag_bank_sync')
        elif stage.startswith('pack_'):
            tag_id = request.env.ref('openerp_enterprise.tag_task_partner_%s' % stage)

        if tag_id:
            values['tag_ids'].extend([Command.link(tag_id.id)])

        # Check for attachments
        files = [kwargs[f] for f in kwargs.keys() if f.startswith('attachment') and kwargs[f]]
        if len(files):
            kwargs['attachments'] = str(len(files))

        # Put the contextual fields content in the description
        contextual_description = reduce(
            lambda desc, f: desc + '\n' + f.replace('_', ' ').capitalize() + ': ' + kwargs[f] if kwargs.get(f) else desc,
            contextual_fields,
            ''
        )
        values['description'] = tools.plaintext2html(contextual_description + '\n\n' + kwargs['description'])

        ticket_id = None
        ProjectTask = request.env['project.task']
        # Try to match an existing issue with this id to write on it
        if 'ticket_id' in kwargs and request.env.user != request.env.ref('base.public_user'):
            ticket_id = ProjectTask.search([('id', '=', kwargs['ticket_id'])])

        if ticket_id:
            # Write a message with the additional informations instead
            ticket_id.message_post(
                subject='Additional Info from Support Form',
                body='Issue: ' + values['name'] + '\n' + values['description'],
                content_subtype='plaintext',
            )
        else:
            # Fallback on creating a new issue if none is found
            ticket_id = ProjectTask.sudo().create(values)

        # Handle CC field
        followers = kwargs.get('followers')
        if followers:
            emails = [_f for _f in [e.strip() for e in re.split('[, ;]', followers)] if _f]
            partner_ids = [
                p.id
                for p in ticket_id._mail_find_partner_from_emails(
                    emails, records=ticket_id, force_create=True
                )
                if p
            ]
            if partner_ids:
                follower_ids.extend(partner_ids)
        # Allow using a different e-mail when logged in (to create a ticket for someone else)
        if partner_id and kwargs['email'] != partner_id.email:
            partner = ticket_id._mail_find_partner_from_emails(
                [kwargs["email"]], records=ticket_id, force_create=True
            )
            if partner:
                follower_ids.extend([p.id for p in partner])

        follower_ids = [_f for _f in follower_ids if _f]
        if follower_ids:
            ticket_id.sudo().message_subscribe(partner_ids=follower_ids)

        # Create attachments
        for f in files:
            request.env['ir.attachment'].sudo().create({
                'name': f.filename,
                'res_model': 'project.task',
                'res_id': ticket_id.id,
                'mimetype': f.mimetype,
                'raw': f.stream.read(),
            })

        return request.render('openerp_enterprise.support_form_thanks', {
            'ticket_id': ticket_id,
            'type': 'update' if 'ticket_id' in kwargs else 'create',
        })

    @route('/help/check_db_name', type='json', auth='public')
    def check_db_name(self, db_name):
        return bool(self._check_db_name(db_name))

    def _check_db_name(self, db_name):
        db_name = db_name.lower()

        database_domain = [
            '|',
            ('db_name', '=', db_name.replace('.odoo.com', '')),
            ('url', 'in', [
                'http://' + db_name.replace('http://', ''),
                'https://' + db_name.replace('https://', ''),
            ]),
            ('hosting', '!=', 'premise'),
        ]
        databases = request.env['openerp.enterprise.database'].sudo().search(database_domain)
        return databases.ids
